# Scoped Storage in React Native

File system access is an essential part of making an application. Up until Android 10 any app could request for storage permissions and have access to all files stored on user's device. Most apps require access to just one folder on user's device. There are only few apps like file managers or backup tools that need access to all user storage. This posed a huge risk on user privacy. Hence Google decided to change this.

Since the release of Android 10, Google has made Scoped storage the default route to access files on user's device. Up until android 11, it was possible to override this with `android:requestLegacyExternalStorage="true"` in `AndroidManifest.xml` but now google is enforcing all apps to use Scoped Storage if they need to store files on user's device. [Some apps can still get all files access if the pass this criteria](https://developer.android.com/training/data-storage/manage-all-files).

## How scoped storage works?

Scoped storage brings the user in control of their device storage. Whenever an app needs to write files on user's device, the user must explicitly grant permission to store files to a specific folder. The app can then read/write to that folder and it's subdirectories only.

## Android Scoped Storage for React Native

At the start of 2021 I came across [Google's policy update](https://developer.android.com/about/versions/11/privacy/storage#scoped-storage):

> Apps that run on Android 11 but target Android 10 (API level 29) can still request the `requestLegacyExternalStorage` attribute. This flag allows apps to temporarily opt out of the changes associated with scoped storage, such as granting access to different directories and different types of media files. **After you update your app to target Android 11, the system ignores the `requestLegacyExternalStorage` flag.**

Our app requires access to device storage to save backup files, note attachments, recovery keys etc. I knew that soon we will need to migrate from legacy storage on android.

After a quick search on Google and Github I was unable to find any library or an example on how scoped storage could be implemented. Hence I decided to create our own library for this and make it open source.

[react-native-scoped-storage](https://github.com/ammarahm-ed/react-native-scoped-storage) tries to solve the scoped storage problem in React Native with an easy to use and well documented API. We were able to easily migrate to using scoped storage for all our use cases in just a few days. So let's see how this works

## Getting permissions to read/write files

In our app, we need the user to give us permission to a folder where we can store some files without interrupting the user. We want to ask for permissions only once for a folder on user's device.

First we will request user to give us access to a folder:

```js
import * as ScopedStorage from "react-native-scoped-storage";
import AsyncStorage from "@react-native-async-storage/async-storage";

let dir = await ScopedStorage.openDocumentTree(true);
```

Android limits the number of directories an app can get access too. Although it does not mention how many. Hence we will store the information about this directory in storage for later use instead of asking for permission of other directory again.

```js
if (dir) {
  // Save information about the directory in AsyncStorage.
  await AsyncStorage.setItem("userDataDirectory", JSON.stringify(dir));
}
```

We might need to save some files upon requesting permission.

```js
let text = "hello world";
await ScopedStorage.writeFile(dir.uri, "helloworld.txt", "text/plain", text);
```

Permissions for the folder are now persisted. If we want to save another file later, we can simply fetch information about the directory from AsyncStorage and save our files.

```js
let dir = await AsyncStorage.getItem("userDataDirectory");
dir = JSON.parse(dir);
```

Remember that Android can revoke the permissions anytime. So before we try to write another file in the directory sometime later, we should first check if we still have permission.

```js
const persistedUris = await ScopedStorage.getPersistedUriPermissions(); // List of all uris where the app has access to read/write

if (persistedUris.indexOf(dir.uri) !== -1) {
  // Write our file
} else {
  let dir = await ScopedStorage.openDocumentTree(true);
  // Store the new directory data in Async Storage then save our file
}
```

Since we need to do this repeatedly, we can simply write a function like the one below to save us the hassle and simplify our workflow:

```ts
import * as ScopedStorage from "react-native-scoped-storage";
import AsyncStorage from "@react-native-async-storage/async-storage";

async function requestPermission(id: string) {
  dir = await ScopedStorage.openDocumentTree(true);
  if (!dir) return null; // User cancelled
  await AsyncStorage.setItem(id, JSON.stringify(dir));
  return dir;
}

export async function getAndroidDir(id: string) {
  try {
    let dir = await AsyncStorage.getItem(id); // Check if dir exists already

    if (dir) {
      dir = JSON.parse(dir);
      const persistedUris = await ScopedStorage.getPersistedUriPermissions(); // list all persisted uris
      if (persistedUris.indexOf(dir.uri) !== -1) return dir; // Verify we still have permission
      return await requestPermission(id); // request new permissions & save the dir;
    } else {
      return await requestPermission(id); // request new permissions & save the dir;
    }
  } catch (e) {
    console.log(e);
    return null;
  }
}
```

## Revoking permissions to a specific folder

If you need to revoke access to specific folders you can do that as follows:

```js
await ScopedStorage.releasePersistableUriPermission(dir.uri);
```

## Directories you can no longer access

Remember that you cannot get permission to read/write in the following directories.

- The root directory of the internal storage volume.
- The root directory of each SD card volume that the device manufacturer considers to be reliable, regardless of whether the card is emulated or removable. A reliable volume is one that an app can successfully access most of the time.
- The Download directory.

source: [developer.android.com](https://developer.android.com/about/versions/11/privacy/storage#directory-access)

## Asking user to save a file

You can save a file after requesting permissions to a folder above. But that is required if you need to save something in background or without opening file manager. But in some cases you just need to save a file and its great if user can decide where it is stored. An example use case is saving a file to Downloads folder or root of the app.

```js
let file = await ScopedStorage.createDocument(
  "helloworld.txt",
  "text/plain",
  "hello world"
);
```

User can save the file on their cloud storage through this. For example Dropbox or Google Drive.

## Asking user to select a file

We can also select a file from storage and read it's contents with scoped storage.

```js
let file = await ScopedStorage.openDocument();
```

## Conclusion

Prior to Android 10 it was really simple to read/write data on user's device. However due to policy changes by Google, we will need migrate our apps to use Scoped Storage so that we can prevent our apps from a broken user experience. However this access will be limited to specific folders on user's device.

You can play around with the exaxmple app and checkout the complete documentation of the library on [Github repo](https://github.com/ammarahm-ed/react-native-scoped-storage).
